@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -58,8 +58,8 @@ import java.util.concurrent.atomic.AtomicBoolean;
 import com.openexchange.mail.MailException;
 import com.openexchange.mail.api.MailAccess;
 import com.openexchange.mail.dataobjects.MailFolder;
-import com.openexchange.mail.dataobjects.ReadOnlyMailFolder;
 import com.openexchange.mail.dataobjects.MailFolder.DefaultFolderType;
+import com.openexchange.mail.dataobjects.ReadOnlyMailFolder;
 import com.openexchange.mail.permission.DefaultMailPermission;
 import com.openexchange.mail.permission.MailPermission;
 import com.openexchange.mailaccount.MailAccount;
@@ -69,10 +69,9 @@ import com.openexchange.mailaccount.UnifiedINBOXManagement;
 import com.openexchange.server.ServiceException;
 import com.openexchange.server.impl.OCLPermission;
 import com.openexchange.session.Session;
-import com.openexchange.threadpool.ThreadPoolService;
 import com.openexchange.threadpool.ThreadPools;
-import com.openexchange.unifiedinbox.UnifiedINBOXAccess;
-import com.openexchange.unifiedinbox.UnifiedINBOXException;
+import com.openexchange.unifiedinbox.UnifiedInboxAccess;
+import com.openexchange.unifiedinbox.UnifiedInboxException;
 import com.openexchange.unifiedinbox.services.UnifiedINBOXServiceRegistry;
 import com.openexchange.unifiedinbox.utility.LoggingCallable;
 import com.openexchange.unifiedinbox.utility.TrackingCompletionService;
@@ -89,7 +88,7 @@ public final class UnifiedINBOXFolderConverter {
     static final int[] EMPTY_COUNTS = new int[] { 0, 0, 0, 0 };
 
     private static final org.apache.commons.logging.Log LOG =
-        org.apache.commons.logging.LogFactory.getLog(UnifiedINBOXFolderConverter.class);
+        com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(UnifiedINBOXFolderConverter.class));
 
     private static final MailFolder ROOT_UNIFIED_INBOX_FOLDER;
 
@@ -100,7 +99,7 @@ public final class UnifiedINBOXFolderConverter {
         tmp.setRootFolder(true);
         tmp.setExists(true);
         tmp.setSeparator('/');
-        // Only the default folder contains subfolders, to be more precise it only contains the INBOX folder.
+        // Only the default folder contains subfolders
         tmp.setSubfolders(true);
         tmp.setSubscribedSubfolders(true);
         tmp.setFullname(MailFolder.DEFAULT_FOLDER_ID);
@@ -154,13 +153,13 @@ public final class UnifiedINBOXFolderConverter {
      * 
      * @param unifiedInboxAccountId The account ID of the Unified INBOX account
      * @param session The session
-     * @param fullname The folder's fullname
+     * @param fullName The folder's full name
      * @param localizedName The localized name of the folder
      * @return The appropriately filled instance of {@link MailFolder}
      * @throws MailException If converting mail folder fails
      */
-    public static MailFolder getUnifiedINBOXFolder(final int unifiedInboxAccountId, final Session session, final String fullname, final String localizedName) throws MailException {
-        return getUnifiedINBOXFolder(unifiedInboxAccountId, session, fullname, localizedName, null);
+    public static MailFolder getUnifiedINBOXFolder(final int unifiedInboxAccountId, final Session session, final String fullName, final String localizedName) throws MailException {
+        return getUnifiedINBOXFolder(unifiedInboxAccountId, session, fullName, localizedName, null);
     }
 
     /**
@@ -168,13 +167,13 @@ public final class UnifiedINBOXFolderConverter {
      * 
      * @param unifiedInboxAccountId The account ID of the Unified INBOX account
      * @param session The session
-     * @param fullname The folder's fullname
+     * @param fullName The folder's full name
      * @param localizedName The localized name of the folder
      * @param executor The executor to use to concurrently load accounts' message counts
      * @return The appropriately filled instance of {@link MailFolder}
      * @throws MailException If converting mail folder fails
      */
-    public static MailFolder getUnifiedINBOXFolder(final int unifiedInboxAccountId, final Session session, final String fullname, final String localizedName, final Executor executor) throws MailException {
+    public static MailFolder getUnifiedINBOXFolder(final int unifiedInboxAccountId, final Session session, final String fullName, final String localizedName, final Executor executor) throws MailException {
         final MailFolder tmp = new MailFolder();
         // Subscription not supported by Unified INBOX, so every folder is "subscribed"
         tmp.setSubscribed(true);
@@ -182,43 +181,44 @@ public final class UnifiedINBOXFolderConverter {
         tmp.setRootFolder(false);
         tmp.setExists(true);
         tmp.setSeparator('/');
-        tmp.setFullname(fullname);
+        tmp.setFullname(fullName);
         tmp.setParentFullname(MailFolder.DEFAULT_FOLDER_ID);
         tmp.setName(localizedName);
         tmp.setHoldsFolders(true);
         tmp.setHoldsMessages(true);
         {
             final MailPermission ownPermission = new DefaultMailPermission();
-            ownPermission.setFolderPermission(OCLPermission.CREATE_OBJECTS_IN_FOLDER);
-            ownPermission.setAllObjectPermission(OCLPermission.READ_ALL_OBJECTS, OCLPermission.NO_PERMISSIONS, OCLPermission.NO_PERMISSIONS);
+            ownPermission.setFolderPermission(OCLPermission.READ_FOLDER);
+            ownPermission.setAllObjectPermission(OCLPermission.READ_ALL_OBJECTS, OCLPermission.NO_PERMISSIONS, OCLPermission.DELETE_ALL_OBJECTS);
+            ownPermission.setFolderAdmin(false);
             tmp.setOwnPermission(ownPermission);
         }
         {
             final MailPermission permission = new DefaultMailPermission();
             permission.setEntity(OCLPermission.ALL_GROUPS_AND_USERS);
             permission.setGroupPermission(true);
-            permission.setFolderPermission(OCLPermission.CREATE_OBJECTS_IN_FOLDER);
-            permission.setAllObjectPermission(OCLPermission.READ_ALL_OBJECTS, OCLPermission.NO_PERMISSIONS, OCLPermission.NO_PERMISSIONS);
+            permission.setFolderPermission(OCLPermission.READ_FOLDER);
+            permission.setAllObjectPermission(OCLPermission.READ_ALL_OBJECTS, OCLPermission.NO_PERMISSIONS, OCLPermission.DELETE_ALL_OBJECTS);
             permission.setFolderAdmin(false);
             tmp.addPermission(permission);
         }
         // What else?!
         tmp.setDefaultFolder(true);
-        if (UnifiedINBOXAccess.INBOX.equals(fullname)) {
+        if (UnifiedInboxAccess.INBOX.equals(fullName)) {
             tmp.setDefaultFolderType(DefaultFolderType.INBOX);
-        } else if (UnifiedINBOXAccess.TRASH.equals(fullname)) {
+        } else if (UnifiedInboxAccess.TRASH.equals(fullName)) {
             tmp.setDefaultFolderType(DefaultFolderType.TRASH);
-        } else if (UnifiedINBOXAccess.SENT.equals(fullname)) {
+        } else if (UnifiedInboxAccess.SENT.equals(fullName)) {
             tmp.setDefaultFolderType(DefaultFolderType.SENT);
-        } else if (UnifiedINBOXAccess.SPAM.equals(fullname)) {
+        } else if (UnifiedInboxAccess.SPAM.equals(fullName)) {
             tmp.setDefaultFolderType(DefaultFolderType.SPAM);
-        } else if (UnifiedINBOXAccess.DRAFTS.equals(fullname)) {
+        } else if (UnifiedInboxAccess.DRAFTS.equals(fullName)) {
             tmp.setDefaultFolderType(DefaultFolderType.DRAFTS);
         } else {
             tmp.setDefaultFolderType(DefaultFolderType.NONE);
         }
         // Set message counts
-        final boolean hasAtLeastOneSuchFolder = setMessageCounts(fullname, unifiedInboxAccountId, session, tmp, executor);
+        final boolean hasAtLeastOneSuchFolder = setMessageCounts(fullName, unifiedInboxAccountId, session, tmp, executor);
         if (hasAtLeastOneSuchFolder) {
             tmp.setSubfolders(true);
             tmp.setSubscribedSubfolders(true);
@@ -226,30 +226,50 @@ public final class UnifiedINBOXFolderConverter {
         return tmp;
     }
 
-    private static boolean setMessageCounts(final String fullname, final int unifiedInboxAccountId, final Session session, final MailFolder tmp, final Executor executor) throws UnifiedINBOXException, MailException {
+    public static void setOwnPermission(final MailFolder mailFolder, final int userId) {
+        final MailPermission ownPermission = new DefaultMailPermission();
+        ownPermission.setEntity(userId);
+        ownPermission.setGroupPermission(false);
+        ownPermission.setFolderPermission(OCLPermission.CREATE_OBJECTS_IN_FOLDER);
+        ownPermission.setAllObjectPermission(OCLPermission.READ_ALL_OBJECTS, OCLPermission.NO_PERMISSIONS, OCLPermission.DELETE_ALL_OBJECTS);
+        ownPermission.setFolderAdmin(false);
+        mailFolder.setOwnPermission(ownPermission);
+    }
+
+    public static void setPermissions(final MailFolder mailFolder) {
+        final MailPermission permission = new DefaultMailPermission();
+        permission.setEntity(OCLPermission.ALL_GROUPS_AND_USERS);
+        permission.setGroupPermission(true);
+        permission.setFolderPermission(OCLPermission.CREATE_OBJECTS_IN_FOLDER);
+        permission.setAllObjectPermission(OCLPermission.READ_ALL_OBJECTS, OCLPermission.NO_PERMISSIONS, OCLPermission.DELETE_ALL_OBJECTS);
+        permission.setFolderAdmin(false);
+        mailFolder.removePermissions();
+        mailFolder.addPermission(permission);
+    }
+
+    private static boolean setMessageCounts(final String fullName, final int unifiedInboxAccountId, final Session session, final MailFolder tmp, final Executor executor) throws UnifiedInboxException, MailException {
         final MailAccount[] accounts;
         try {
             final MailAccountStorageService storageService =
                 UnifiedINBOXServiceRegistry.getServiceRegistry().getService(MailAccountStorageService.class, true);
             final MailAccount[] arr = storageService.getUserMailAccounts(session.getUserId(), session.getContextId());
             final List<MailAccount> l = new ArrayList<MailAccount>(arr.length);
-            for (int i = 0; i < arr.length; i++) {
-                final MailAccount mailAccount = arr[i];
+            for (final MailAccount mailAccount : arr) {
                 if (unifiedInboxAccountId != mailAccount.getId() && mailAccount.isUnifiedINBOXEnabled()) {
                     l.add(mailAccount);
                 }
             }
             accounts = l.toArray(new MailAccount[l.size()]);
         } catch (final ServiceException e) {
-            throw new UnifiedINBOXException(e);
+            throw new UnifiedInboxException(e);
         } catch (final MailAccountException e) {
-            throw new UnifiedINBOXException(e);
+            throw new UnifiedInboxException(e);
         }
         // Create completion service for simultaneous access
         final int length = accounts.length;
         final Executor exec;
         if (executor == null) {
-            exec = UnifiedINBOXServiceRegistry.getServiceRegistry().getService(ThreadPoolService.class).getExecutor();
+            exec = ThreadPools.getThreadPool().getExecutor();
         } else {
             exec = executor;
         }
@@ -263,16 +283,11 @@ public final class UnifiedINBOXFolderConverter {
                 completionService.submit(new LoggingCallable<int[]>(session) {
 
                     public int[] call() throws Exception {
-                        final MailAccess<?, ?> mailAccess;
+                        MailAccess<?, ?> mailAccess = null;
                         try {
                             mailAccess = MailAccess.getInstance(session, mailAccount.getId());
                             mailAccess.connect();
-                        } catch (final MailException e) {
-                            getLogger().error(e.getMessage(), e);
-                            return EMPTY_COUNTS;
-                        }
-                        try {
-                            final String accountFullname = UnifiedINBOXUtility.determineAccountFullname(mailAccess, fullname);
+                            final String accountFullname = UnifiedINBOXUtility.determineAccountFullname(mailAccess, fullName);
                             // Check if account fullname is not null
                             if (null == accountFullname) {
                                 return EMPTY_COUNTS;
@@ -285,8 +300,13 @@ public final class UnifiedINBOXFolderConverter {
                                     mailFolder.getNewMessageCount() };
                             retval.set(true);
                             return counts;
+                        } catch (final MailException e) {
+                            getLogger().debug(e.getMessage(), e);
+                            return EMPTY_COUNTS;
                         } finally {
-                            mailAccess.close(true);
+                            if (null != mailAccess) {
+                                mailAccess.close(true);
+                            }
                         }
                     }
                 });
@@ -309,7 +329,7 @@ public final class UnifiedINBOXFolderConverter {
                 newCount += counts[3];
             }
             if (LOG.isDebugEnabled()) {
-                LOG.debug(new StringBuilder("Retrieving message counts of folder \"").append(fullname).append("\" took ").append(
+                LOG.debug(new StringBuilder("Retrieving message counts of folder \"").append(fullName).append("\" took ").append(
                     completionService.getDuration()).append("msec."));
             }
             // Apply counts
@@ -351,15 +371,10 @@ public final class UnifiedINBOXFolderConverter {
     private static int[][] getAccountDefaultFolders0(final int accountId, final Session session, final String[] fullnames) throws MailException {
         final int[][] retval = new int[fullnames.length][];
         // Get & connect appropriate mail access
-        final MailAccess<?, ?> mailAccess;
+        MailAccess<?, ?> mailAccess = null;
         try {
             mailAccess = MailAccess.getInstance(session, accountId);
             mailAccess.connect();
-        } catch (final MailException e) {
-            LOG.error(e.getMessage(), e);
-            return new int[0][];
-        }
-        try {
             for (int i = 0; i < retval.length; i++) {
                 final String accountFullname = UnifiedINBOXUtility.determineAccountFullname(mailAccess, fullnames[i]);
                 if (null != accountFullname && mailAccess.getFolderStorage().exists(accountFullname)) {
@@ -371,8 +386,13 @@ public final class UnifiedINBOXFolderConverter {
                         accountId).toString());
                 }
             }
+        } catch (final MailException e) {
+            LOG.error(e.getMessage(), e);
+            return new int[0][];
         } finally {
-            mailAccess.close(true);
+            if (null != mailAccess) {
+                mailAccess.close(true);
+            }
         }
         return retval;
     }
@@ -413,15 +433,15 @@ public final class UnifiedINBOXFolderConverter {
             }
             // What else?!
             tmp.setDefaultFolder(true);
-            if (UnifiedINBOXAccess.INBOX.equals(fullname)) {
+            if (UnifiedInboxAccess.INBOX.equals(fullname)) {
                 tmp.setDefaultFolderType(DefaultFolderType.INBOX);
-            } else if (UnifiedINBOXAccess.TRASH.equals(fullname)) {
+            } else if (UnifiedInboxAccess.TRASH.equals(fullname)) {
                 tmp.setDefaultFolderType(DefaultFolderType.TRASH);
-            } else if (UnifiedINBOXAccess.SENT.equals(fullname)) {
+            } else if (UnifiedInboxAccess.SENT.equals(fullname)) {
                 tmp.setDefaultFolderType(DefaultFolderType.SENT);
-            } else if (UnifiedINBOXAccess.SPAM.equals(fullname)) {
+            } else if (UnifiedInboxAccess.SPAM.equals(fullname)) {
                 tmp.setDefaultFolderType(DefaultFolderType.SPAM);
-            } else if (UnifiedINBOXAccess.DRAFTS.equals(fullname)) {
+            } else if (UnifiedInboxAccess.DRAFTS.equals(fullname)) {
                 tmp.setDefaultFolderType(DefaultFolderType.DRAFTS);
             } else {
                 tmp.setDefaultFolderType(DefaultFolderType.NONE);
